using System;
using System.IO;
using System.Drawing;
using System.Reflection;
using System.Collections;
using System.Collections.Generic;

using SdlDotNet.Core;
using SdlDotNet.Graphics;
using SdlDotNet.Graphics.Sprites;
using SdlDotNet.Input;

using BoO.lib.obj.interfaces;

namespace BoO.lib.obj
{
    class character : ICoord
    {
        protected AnimatedSprite _anim;
        protected string direction = null, name = null, id = null;

        protected int hp = 0, mp = 0;

        protected bool playable = false;

        protected Point _vcoord, _coord;
        protected Rectangle _vrec, _rec;

        public Rectangle _tmp_col_check;
        public int speed = 0;

        public character(string na, string id, int hp, int mp, int sp, int alpha, bool playable, Point coord)
        {
            this.name = na;
            this.id = id;
            this.hp = hp;
            this.mp = mp;
            speed = sp;
            this.playable = playable;
            this._coord = this._vcoord = coord;

            _anim = new AnimatedSprite();
            _anim.Animate = true;
            _anim.Position = _coord;

            if (playable == true)
                BoO.share.main_char_speed = speed;

            this.load_images(alpha);
        }

        private void load_images(int alpha)
        {
            string char_img_dir = BoO_Config.Main.PROGRAM_ROOT_PATH + "/" + BoO_Config.Main.char_dir + "/" + this.id;

            if (!Directory.Exists(char_img_dir))
            {
                Events.QuitApplication();
            }

            DirectoryInfo dir = new DirectoryInfo(char_img_dir);
            foreach (DirectoryInfo sdi in dir.GetDirectories())
            {
                SurfaceCollection sdir_sc = new SurfaceCollection();
                string sdir = sdi.Name;

                DirectoryInfo ssdir = new DirectoryInfo(char_img_dir + "/" + sdir);
                int i = 0;
                foreach (FileInfo ssdi in ssdir.GetFiles())
                {
                    Surface tmp = new Surface(ssdi.FullName).Convert(Video.Screen);
                    if (alpha < 255)
                    {
                        tmp.AlphaBlending = true;
                        tmp.Alpha = Convert.ToByte(alpha);
                    }
                    tmp.TransparentColor = BoO_Config.Video.transparent_color;
                    tmp.Transparent = true;
                    sdir_sc.Add(tmp);

                    if (i > 0)
                        sdir_sc.Add(tmp);

                    i++;
                }

                Console.WriteLine("Found " + sdir_sc.Count + " Animations for Animation " + sdir);

                if(sdir_sc.Count > 0)
                    this.set_anim(sdir, sdir_sc, Convert.ToInt32(1000 / sdir_sc.Count));
            }
        }

        public void set_anim(String anim, SurfaceCollection surcol, int delay)
        {
            AnimationCollection ac = new AnimationCollection();
            ac.Add(surcol);
            ac.Delay = delay;
            _anim.Animations.Add(anim, ac);
        }

        public void set_cur_anim(String anim)
        {
            _anim.CurrentAnimation = anim;
            direction = anim;
        }

        public void set_anim(bool anim)
        {
            _anim.Animate = anim;
        }

        public void move(Dictionary<string, bool> keys)
        {
            if (keys["left"] == true)
            {
                // Check if char collides with object
                _tmp_col_check = new Rectangle(new Point(_vcoord.X - speed, _vcoord.Y), _anim.Size);
                if (collision.collide(_tmp_col_check, level.collides))
                    goto Buttons;

                // Check if char is on end of map
                if (_coord.X - speed < 0)
                    goto Buttons;

                // When Background is not scrolled, update char position
                if (BoOevents.events.bg_update == false)
                    update_coord(-speed, 0);

                update_vcoord(-speed, 0);

                if (direction != "left")
                    set_cur_anim("left");
            }
            else if (keys["right"] == true)
            {
                _tmp_col_check = new Rectangle(new Point(_vcoord.X + speed, _vcoord.Y), _anim.Size);
                if (collision.collide(_tmp_col_check, level.collides))
                    goto Buttons;

                if (_coord.X + speed + _anim.Rectangle.Width > BoO_Config.Video.res_h)
                    goto Buttons;

                if (BoOevents.events.bg_update == false)
                    update_coord(speed, 0);

                update_vcoord(speed, 0);

                if (direction != "right")
                    set_cur_anim("right");
            }
            else if (keys["up"] == true)
            {
                _tmp_col_check = new Rectangle(new Point(_vcoord.X, _vcoord.Y - speed), _anim.Size);
                if (collision.collide(_tmp_col_check, level.collides))
                    goto Buttons;

                if (_coord.Y - speed < 0)
                    goto Buttons;

                if (BoOevents.events.bg_update == false)
                    update_coord(0, -speed);

                if (_vcoord.Y - speed >= 0)
                    update_vcoord(0, -speed);

                if (direction != "up")
                    set_cur_anim("up");
            }
            else if (keys["down"] == true)
            {
                _tmp_col_check = new Rectangle(new Point(_vcoord.X, _vcoord.Y + speed), _anim.Size);
                if (collision.collide(_tmp_col_check, level.collides))
                    goto Buttons;

                if (_coord.Y + speed + _anim.Rectangle.Height > BoO_Config.Video.res_v)
                    goto Buttons;

                if (BoOevents.events.bg_update == false)
                    update_coord(0, speed);

                if (_vcoord.Y + speed + _anim.Rectangle.Height <= BoO.share.map_size.Height)
                    update_vcoord(0, speed);

                //Console.WriteLine(direction);

                if (direction != "down")
                    set_cur_anim("down");
            }
            else
            {
                
                if (direction != "stand")
                    set_cur_anim("stand");
                //_anim.Frame = 0;
            }

            Buttons:
            if (keys["button_1"] == true)
            {
                Console.WriteLine("button_1 pressed");

                _tmp_col_check = new Rectangle(new Point(_vcoord.X, _vcoord.Y - speed), _anim.Size);
                objInteractive objI = collision.collide_with_objInteractive_as_char(_tmp_col_check);
                if (objI != null)
                {
                    objI.on_button1_press();
                }
            }

            _anim.Position = _coord;
            if (playable == true)
            {
                BoO.share.main_char = _anim.Rectangle;
                BoO.share.main_char_vcoord = _vcoord;
            }
        }

        public void render()
        {
            Video.Screen.Blit(_anim, _coord);
        }

        #region ICoord
        public void update_coord(int x, int y)
        {
            _coord.X += x;
            _coord.Y += y;

            if (_coord != null && _anim != null)
                _rec = new Rectangle(_coord, _anim.Size);
        }

        public void update_coord(Point p)
        {
            update_coord(p.X, p.Y);
        }

        public void update_vcoord(int x, int y)
        {
            _vcoord.X += x;
            _vcoord.Y += y;

            if (_vcoord != null && _anim != null)
                _vrec = new Rectangle(_vcoord, _anim.Size);
        }

        public void update_vcoord(Point p)
        {
            update_vcoord(p.X, p.Y);
        }

        public void sett_coord(int x, int y)
        {
            _coord.X = x;
            _coord.Y = y;

            if (_coord != null && _anim != null)
                _rec = new Rectangle(_coord, _anim.Size);
        }

        public void sett_coord(Point p)
        {
            sett_coord(p.X, p.Y);
        }

        public void sett_vcoord(int x, int y)
        {
            _vcoord.X = x;
            _vcoord.Y = y;

            if (_vcoord != null && _anim != null)
                _vrec = new Rectangle(_vcoord, _anim.Size);
        }

        public void sett_vcoord(Point p)
        {
            sett_vcoord(p.X, p.Y);
        }

        public Point coord
        {
            get { return _coord; }
        }

        public Point vcoord
        {
            get { return _vcoord; }
        }

        public Rectangle rec
        {
            get { return _rec; }
        }

        public Rectangle vrec
        {
            get { return _vrec; }
        }
        #endregion ICoord
    }
}
