using System;
using System.IO;
using System.Drawing;
using System.Reflection;
using System.Collections;
using System.Collections.Generic;
using System.Text.RegularExpressions;

using SdlDotNet.Core;
using SdlDotNet.Graphics;
using SdlDotNet.Graphics.Sprites;
using SdlDotNet.Input;

using BoO.lib.obj;

namespace BoO.lib.obj
{
    class level
    {
        private string data_dir = null;

        private Dictionary<string, bool> _keys;
        private List<Rectangle> ud_collides = new List<Rectangle>();

        // Images characters can go on
        public static SortedDictionary<int, background> _background_images = new SortedDictionary<int, background>();
        // Objects characters can collide with
        public static SortedDictionary<string, objStatic> _static_images = new SortedDictionary<string, objStatic>();
        // Images above characters
        public static SortedDictionary<string, objAbove> _above_images = new SortedDictionary<string, objAbove>();
        // Objects characters can interact with
        public static SortedDictionary<string, objInteractive> _interact_images = new SortedDictionary<string, objInteractive>();
        // Characters in this level
        public static SortedDictionary<string, character> _character_images = new SortedDictionary<string, character>();

        public static List<Rectangle> collides = new List<Rectangle>();
        public static string name = null;

        public level(string n)
        {
            name = n;

            load_level();
        }

        public void update_main()
        {
            BoO.lib.obj.level.collides.Clear();

            foreach (Rectangle rec in ud_collides)
            {
                collides.Add(rec);
            }
        }

        #region backgrounds
        public void set_background(int nr, background bg)
        {
            Video.Screen.Blit(new Surface(bg.surface));
            BoO.share.map_size = bg.surface.Rectangle;
            _background_images.Add(nr, bg);
        }

        public void set_backgrounds(SortedDictionary<int, background> dict)
        {
            BoO.share.map_size = dict[0].surface.Rectangle;
            foreach(int nr in dict.Keys)
            {
                set_background(nr, dict[nr]);
            }
        }

        public void update_background()
        {
            background bg;

            foreach (int nr in _background_images.Keys)
            {
                bg = _background_images[nr];
                bg.move(_keys);
            }
        }

        public void render_background()
        {
            background bg;

            foreach (int nr in _background_images.Keys)
            {
                bg = _background_images[nr];
                bg.render();
            }
        }
        #endregion backgrounds

        #region characters
        public void set_character(string id, character ch)
        {
            _character_images.Add(id, ch);
        }

        public void set_characters(SortedDictionary<string, character> dict)
        {
            foreach (string id in dict.Keys)
            {
                set_character(id, dict[id]);
            }
        }

        public void update_characters()
        {
            character ch;

            foreach (string id in _character_images.Keys)
            {
                ch = _character_images[id];
                ch.move(_keys);
            }
        }

        public void render_characters()
        {
            character ch;

            foreach (string id in _character_images.Keys)
            {
                ch = _character_images[id];
                ch.render();
            }
        }
        #endregion characters

        #region interactiveobj
        public void set_objInteractive(string id, objInteractive oi)
        {
            _interact_images.Add(id, oi);
        }

        public void set_objsInteractive(SortedDictionary<string, objInteractive> dict)
        {
            foreach (string id in dict.Keys)
            {
                set_objInteractive(id, dict[id]);
            }
        }

        public void update_objsInteractive()
        {
            foreach (string id in _interact_images.Keys)
            {
                _interact_images[id].move(_keys);
            }
        }

        public void render_objsInteractive()
        {
            objInteractive oi;

            foreach (string id in _interact_images.Keys)
            {
                oi = _interact_images[id];
                oi.render();
            }
        }
        #endregion interactiveobj

        #region aboveobj
        public void set_objAbove(string id, objAbove oi)
        {
            _above_images.Add(id, oi);
        }

        public void set_objsAbove(SortedDictionary<string, objAbove> dict)
        {
            foreach (string id in dict.Keys)
            {
                set_objAbove(id, dict[id]);
            }
        }

        public void update_objsAbove()
        {
            foreach (string id in _above_images.Keys)
            {
                _above_images[id].move(_keys);
            }
        }

        public void render_objsAbove()
        {
            objAbove oi;

            foreach (string id in _above_images.Keys)
            {
                oi = _above_images[id];
                oi.render();
            }
        }
        #endregion aboveobj

        #region staticobj
        public void set_objStatic(string id, objStatic oi)
        {
            _static_images.Add(id, oi);
        }

        public void set_objsStatic(SortedDictionary<string, objStatic> dict)
        {
            foreach (string id in dict.Keys)
            {
                set_objStatic(id, dict[id]);
            }
        }

        public void update_objsStatic()
        {
            foreach (string id in _static_images.Keys)
            {
                _static_images[id].move(_keys);
            }
        }

        public void render_objsStatic()
        {
            objStatic oi;

            foreach (string id in _static_images.Keys)
            {
                oi = _static_images[id];
                oi.render();
            }
        }
        #endregion staticobj

        #region setsgets

        public Dictionary<string, bool> keys
        {
            get
            {
                return _keys;
            }

            set
            {
                _keys = value;
            }
        }
        #endregion setsgets

        #region loadleveldata
        private void load_level()
        {
            string map_file = BoO_Config.Main.PROGRAM_ROOT_PATH + "/" + BoO_Config.Main.map_dir + "/" + name + ".map", buf = null, type = null;
            bool read = false;

            if (!File.Exists(map_file))
            {
                Events.QuitApplication();
            }

            StreamReader tr = new StreamReader(map_file);
            while (!tr.EndOfStream)
            {
                string str = tr.ReadLine();

                if (str.StartsWith("#"))
                    continue;

                if (str.Equals(""))
                {
                    if(buf != null)
                        load_level_data(type, buf.Trim());
                    buf = null;
                    type = null;
                    read = false;
                }

                if (read == true)
                {
                    buf += str + "\n";
                    continue;
                }

                if (str.Equals("[Main]"))
                {
                    type = "main";
                    read = true;
                }
                else if (str.Equals("[Background]"))
                {
                    type = "background";
                    read = true;
                }
                else if (str.Equals("[Character]"))
                {
                    type = "character";
                    read = true;
                }
                else if (str.Equals("[aObject]"))
                {
                    type = "aobject";
                    read = true;
                }
                else if (str.Equals("[iObject]"))
                {
                    type = "iobject";
                    read = true;
                }
                else if (str.Equals("[hObject]"))
                {
                    type = "hobject";
                    read = true;
                }
                else if (str.Equals("[sObject]"))
                {
                    type = "sobject";
                    read = true;
                }
                else if (str.Equals("[Collide]"))
                {
                    type = "collide";
                    read = true;
                }
            }
            tr.Close();
        }

        private void load_level_data(string type, string data)
        {
            Regex re_hp = new Regex(@"^HP = ([0-9]+)$", RegexOptions.Compiled);
            Regex re_mp = new Regex(@"^MP = ([0-9]+)$", RegexOptions.Compiled);
            Regex re_id = new Regex(@"^ID = ([a-zA-Z0-9\\.\\_]+)$", RegexOptions.Compiled);
            Regex re_name = new Regex(@"^Name = ([a-zA-Z0-9\\.\\_]+)$", RegexOptions.Compiled);
            Regex re_alpha = new Regex(@"^Alpha = ([0-9]+)$", RegexOptions.Compiled);
            Regex re_image = new Regex(@"^Image = ([a-zA-Z0-9\\.\\_\\/]+)$", RegexOptions.Compiled);
            Regex re_playable = new Regex(@"^Playable = (true|false)$", RegexOptions.Compiled);
            Regex re_collide = new Regex(@"^Collide = (true|false)$", RegexOptions.Compiled);
            Regex re_animate = new Regex(@"^Animate = (true|false)$", RegexOptions.Compiled);
            Regex re_reverse = new Regex(@"^Reverse = (true|false)$", RegexOptions.Compiled);
            Regex re_repeat = new Regex(@"^Repeat = (right|left|up|down),([0-9]+)$", RegexOptions.Compiled);
            Regex re_animatedelay = new Regex(@"^AnimateDelay = ([0-9]+)$", RegexOptions.Compiled);
            Regex re_position = new Regex(@"^Position = ([0-9]+),([0-9]+)$", RegexOptions.Compiled);
            Regex re_rectangle = new Regex(@"^Rectangle = ([0-9]+),([0-9]+),([0-9]+),([0-9]+)$", RegexOptions.Compiled);
            Regex re_colrectangle = new Regex(@"^CollideRectangle = ([0-9]+),([0-9]+),([0-9]+),([0-9]+)$", RegexOptions.Compiled);

            if (type == "main")
            {
                foreach (string line in data.Split("\n".ToCharArray()))
                {
                    if (re_id.IsMatch(line))
                        data_dir = BoO_Config.Main.level_dir + "/" + re_id.Match(line).Groups[1].ToString() + "/";

                }
            }
            else if (type == "background")
            {
                int id = 0, alpha = 255;
                string image = null;

                foreach (string line in data.Split("\n".ToCharArray()))
                {
                    if (re_id.IsMatch(line))
                        id = Convert.ToInt32(re_id.Match(line).Groups[1].ToString());

                    if (re_image.IsMatch(line))
                        image = re_image.Match(line).Groups[1].ToString();

                    if (re_alpha.IsMatch(line))
                        alpha = Convert.ToInt32(re_alpha.Match(line).Groups[1].ToString());
                }

                if (id >= 0 && image != null)
                {
                    Surface tmp = new Surface(data_dir + image).Convert(Video.Screen);

                    if (alpha < 255)
                    {
                        tmp.AlphaBlending = true;
                        tmp.Alpha = Convert.ToByte(alpha);
                    }
                    set_background(id, new background(tmp, BoO_Config.Main.standard_speed));
                }
            }
            else if (type == "character")
            {
                string id = null, name = null;
                Point position = new Point(0, 0);
                int hp = 0, mp = 0, sp = BoO_Config.Main.standard_speed, alpha = 255;
                bool playable = false;

                foreach (string line in data.Split("\n".ToCharArray()))
                {
                    if (re_id.IsMatch(line))
                        id = re_id.Match(line).Groups[1].ToString();

                    if (re_name.IsMatch(line))
                        name = re_name.Match(line).Groups[1].ToString();

                    if (re_hp.IsMatch(line))
                        hp = Convert.ToInt32(re_hp.Match(line).Groups[1].ToString());

                    if (re_mp.IsMatch(line))
                        mp = Convert.ToInt32(re_mp.Match(line).Groups[1].ToString());

                    if (re_playable.IsMatch(line))
                        playable = Convert.ToBoolean(re_playable.Match(line).Groups[1].ToString());

                    if (re_alpha.IsMatch(line))
                        alpha = Convert.ToInt32(re_alpha.Match(line).Groups[1].ToString());

                    if (re_position.IsMatch(line))
                        position = new Point(Convert.ToInt32(re_position.Match(line).Groups[1].ToString()), Convert.ToInt32(re_position.Match(line).Groups[2].ToString()));
                }

                if (id != null && name != null && hp != 0 && mp != 0)
                {
                    Console.WriteLine(">>" + position);

                    character ch = new character(name, id, hp, mp, sp, alpha, playable, position);
                    ch.set_cur_anim("stand");

                    set_character(id, ch);
                }
            }
            else if (type == "hobject")
            {
                string id = null, name = null, repeat_direction = null;
                Point position = new Point();
                Rectangle rec_collide = new Rectangle();
                bool animate = false, reverse = false;
                int animated = 500, repeat_amount = 0, alpha = 255;

                foreach (string line in data.Split("\n".ToCharArray()))
                {
                    if (re_id.IsMatch(line))
                        id = re_id.Match(line).Groups[1].ToString();

                    if (re_name.IsMatch(line))
                        name = re_name.Match(line).Groups[1].ToString();

                    if (re_repeat.IsMatch(line))
                    {
                        repeat_direction = re_repeat.Match(line).Groups[1].ToString();
                        repeat_amount = Convert.ToInt32(re_repeat.Match(line).Groups[2].ToString());
                    }

                    if (re_animate.IsMatch(line))
                        animate = Convert.ToBoolean(re_animate.Match(line).Groups[1].ToString());

                    if (re_reverse.IsMatch(line))
                        reverse = Convert.ToBoolean(re_reverse.Match(line).Groups[1].ToString());

                    if (re_animatedelay.IsMatch(line))
                        animated = Convert.ToInt32(re_animatedelay.Match(line).Groups[1].ToString()) >> 1;

                    if (re_position.IsMatch(line))
                        position = new Point(Convert.ToInt32(re_position.Match(line).Groups[1].ToString()), Convert.ToInt32(re_position.Match(line).Groups[2].ToString()));

                    if (re_colrectangle.IsMatch(line))
                        rec_collide = new Rectangle(Convert.ToInt32(re_colrectangle.Match(line).Groups[1].ToString()), Convert.ToInt32(re_colrectangle.Match(line).Groups[2].ToString()), Convert.ToInt32(re_colrectangle.Match(line).Groups[3].ToString()), Convert.ToInt32(re_colrectangle.Match(line).Groups[4].ToString()));

                    if (re_alpha.IsMatch(line))
                        alpha = Convert.ToInt32(re_alpha.Match(line).Groups[1].ToString());
                }

                if (id != null && name != null)
                {
                    string obj_dir = BoO_Config.Main.PROGRAM_ROOT_PATH + "/" + BoO_Config.Main.level_dir + "/" + level.name + "/obj/" + name;
                    SurfaceCollection sco = new SurfaceCollection();
                    SurfaceCollection sco_c = new SurfaceCollection();

                    if (Directory.Exists(obj_dir) == false)
                    {
                        Events.QuitApplication();
                    }

                    DirectoryInfo di_o = new DirectoryInfo(obj_dir);

                    int i = 0;
                    foreach (FileInfo fi_o in di_o.GetFiles())
                    {
                        Surface s_o = new Surface(fi_o.FullName).Convert(Video.Screen);

                        Surface s_o_c = s_o.CreateSurfaceFromClipRectangle(rec_collide).Convert(Video.Screen);
                        if (alpha < 255)
                        {
                            s_o_c.AlphaBlending = true;
                            s_o_c.Alpha = Convert.ToByte(alpha);
                        }
                        s_o_c.TransparentColor = BoO_Config.Video.transparent_color;
                        s_o_c.Transparent = true;
                        sco_c.Add(s_o_c);
                        if (i > 0)
                            sco_c.Add(s_o_c);

                        for (int x = rec_collide.X; x < (rec_collide.X + rec_collide.Width); x++)
                        {
                            for (int y = rec_collide.Y; y < (rec_collide.Y + rec_collide.Height); y++)
                            {
                                s_o.SetPixels(new Point(x, y), BoO_Config.Video.transparent_color_a);
                            }
                        }

                        if (alpha < 255)
                        {
                            s_o.AlphaBlending = true;
                            s_o.Alpha = Convert.ToByte(alpha);
                        }
                        s_o.TransparentColor = BoO_Config.Video.transparent_color;
                        s_o.Transparent = true;
                        sco.Add(s_o);
                        if (i > 0)
                            sco.Add(s_o);

                        i++;
                    }

                    if (repeat_direction == null)
                    {
                        if (reverse == false)
                        {
                            // Static-collide Part of Object
                            Point position_col = new Point(position.X + rec_collide.X, position.Y + rec_collide.Y);
                            objStatic ois = new objStatic(true, position_col);
                            ois.set_anim("default", sco_c, animated);
                            ois.set_cur_anim("default");
                            ois.set_anim(animate);

                            set_objStatic(id, ois);

                            // Above-non-collide Part of Object
                            objAbove oia = new objAbove(false, position);
                            oia.set_anim("default", sco, animated);
                            oia.set_cur_anim("default");
                            oia.set_anim(animate);

                            set_objAbove(id, oia);
                        }
                        else if (reverse == true)
                        {
                            // Above-non-collide Part of Object
                            Point position_col = new Point(position.X + rec_collide.X, position.Y + rec_collide.Y);
                            objAbove ois = new objAbove(false, position_col);
                            ois.set_anim("default", sco_c, animated);
                            ois.set_cur_anim("default");
                            ois.set_anim(animate);

                            set_objAbove(id, ois);

                            // Static-collide Part of Object
                            objStatic oia = new objStatic(true, position);
                            oia.set_anim("default", sco, animated);
                            oia.set_cur_anim("default");
                            oia.set_anim(animate);

                            set_objStatic(id, oia);
                        }
                    }
                    else
                    {
                        SortedDictionary<string, objAbove> oits = new SortedDictionary<string, objAbove>();
                        SortedDictionary<string, objStatic> oits_c = new SortedDictionary<string, objStatic>();

                        for (int j = 0; j < repeat_amount; j++)
                        {
                            Point tmp_position = position, tmp_position_col = position;
                            tmp_position_col.X = position.X + rec_collide.X;
                            tmp_position_col.Y = position.Y + rec_collide.Y;

                            if (repeat_direction == "right") tmp_position.X += j * sco[0].Width;
                            else if (repeat_direction == "left") tmp_position.X -= j * sco[0].Width;
                            else if (repeat_direction == "down") tmp_position.Y += j * sco[0].Height;
                            else if (repeat_direction == "up") tmp_position.Y -= j * sco[0].Height;

                            if (repeat_direction == "right") tmp_position_col.X += (j * sco[0].Width);
                            else if (repeat_direction == "left") tmp_position_col.X -= (j * sco[0].Width);
                            else if (repeat_direction == "down") tmp_position_col.Y += j * sco[0].Height;
                            else if (repeat_direction == "up") tmp_position_col.Y -= j * sco[0].Height;

                            if (reverse == false)
                            {
                                // Above-non-collide Part of Object
                                objAbove oit = new objAbove(false, tmp_position);
                                oit.set_anim("default", sco, animated);
                                oit.set_cur_anim("default");
                                oit.set_anim(animate);
                                oits.Add(id + "_" + j, oit);

                                // Static-collide Part of Object
                                objStatic oit_c = new objStatic(true, tmp_position_col);
                                oit_c.set_anim("default", sco_c, animated);
                                oit_c.set_cur_anim("default");
                                oit_c.set_anim(animate);
                                oits_c.Add(id + "_" + j, oit_c);
                            }
                            else if (reverse == true)
                            {
                                // Static-collide Part of Object
                                objStatic oit = new objStatic(true, tmp_position);
                                oit.set_anim("default", sco, animated);
                                oit.set_cur_anim("default");
                                oit.set_anim(animate);
                                oits_c.Add(id + "_" + j, oit);

                                // Above-non-collide Part of Object
                                objAbove oit_c = new objAbove(false, tmp_position_col);
                                oit_c.set_anim("default", sco_c, animated);
                                oit_c.set_cur_anim("default");
                                oit_c.set_anim(animate);
                                oits.Add(id + "_" + j, oit_c);
                            }
                        }

                        set_objsAbove(oits);
                        set_objsStatic(oits_c);
                    }
                }
            }
            else if (type == "aobject")
            {
                string id = null, name = null, repeat_direction = null;
                Point position = new Point();
                bool animate = false, collide = false;
                int animated = 500, repeat_amount = 0, alpha = 255;

                foreach (string line in data.Split("\n".ToCharArray()))
                {
                    if (re_id.IsMatch(line))
                        id = re_id.Match(line).Groups[1].ToString();

                    if (re_name.IsMatch(line))
                        name = re_name.Match(line).Groups[1].ToString();

                    if (re_repeat.IsMatch(line))
                    {
                        repeat_direction = re_repeat.Match(line).Groups[1].ToString();
                        repeat_amount = Convert.ToInt32(re_repeat.Match(line).Groups[2].ToString());
                    }

                    if (re_animate.IsMatch(line))
                        animate = Convert.ToBoolean(re_animate.Match(line).Groups[1].ToString());

                    if (re_animatedelay.IsMatch(line))
                        animated = Convert.ToInt32(re_animatedelay.Match(line).Groups[1].ToString()) >> 1;

                    if (re_position.IsMatch(line))
                        position = new Point(Convert.ToInt32(re_position.Match(line).Groups[1].ToString()), Convert.ToInt32(re_position.Match(line).Groups[2].ToString()));

                    if (re_alpha.IsMatch(line))
                        alpha = Convert.ToInt32(re_alpha.Match(line).Groups[1].ToString());
                }

                if (id != null && name != null)
                {
                    string obj_dir = BoO_Config.Main.PROGRAM_ROOT_PATH + "/" + BoO_Config.Main.level_dir + "/" + level.name + "/obj/" + name;
                    SurfaceCollection sco = new SurfaceCollection();

                    if (Directory.Exists(obj_dir) == false)
                    {
                        Events.QuitApplication();
                    }

                    DirectoryInfo di_o = new DirectoryInfo(obj_dir);

                    int i = 0;
                    foreach (FileInfo fi_o in di_o.GetFiles())
                    {
                        Surface s_o = new Surface(fi_o.FullName).Convert(Video.Screen);
                        if (alpha < 255)
                        {
                            s_o.AlphaBlending = true;
                            s_o.Alpha = Convert.ToByte(alpha);
                        }
                        s_o.TransparentColor = BoO_Config.Video.transparent_color;
                        s_o.Transparent = true;
                        sco.Add(s_o);
                        if (i > 0)
                            sco.Add(s_o);

                        i++;
                    }

                    if (repeat_direction == null)
                    {
                        objAbove oit = new objAbove(collide, position);
                        oit.set_anim("default", sco, animated);
                        oit.set_cur_anim("default");
                        oit.set_anim(animate);

                        set_objAbove(id, oit);
                    }
                    else
                    {
                        SortedDictionary<string, objAbove> oits = new SortedDictionary<string, objAbove>();

                        for (int j = 0; j < repeat_amount; j++)
                        {
                            Point tmp_position = position;
                            if (repeat_direction == "right") tmp_position.X += j * sco[0].Width;
                            else if (repeat_direction == "left") tmp_position.X -= j * sco[0].Width;
                            else if (repeat_direction == "down") tmp_position.Y += j * sco[0].Height;
                            else if (repeat_direction == "up") tmp_position.Y -= j * sco[0].Height;

                            objAbove oit = new objAbove(collide, tmp_position);
                            oit.set_anim("default", sco, animated);
                            oit.set_cur_anim("default");
                            oit.set_anim(animate);

                            oits.Add(id + "_" + j, oit);
                        }

                        set_objsAbove(oits);
                    }
                }
            }
            else if (type == "sobject")
            {
                string id = null, name = null, repeat_direction = null;
                Point position = new Point();
                bool collide = true, animate = false;
                int animated = 500, repeat_amount = 0, alpha = 255;

                foreach (string line in data.Split("\n".ToCharArray()))
                {
                    if (re_id.IsMatch(line))
                        id = re_id.Match(line).Groups[1].ToString();

                    if (re_name.IsMatch(line))
                        name = re_name.Match(line).Groups[1].ToString();

                    if (re_repeat.IsMatch(line))
                    {
                        repeat_direction = re_repeat.Match(line).Groups[1].ToString();
                        repeat_amount = Convert.ToInt32(re_repeat.Match(line).Groups[2].ToString());
                    }

                    if (re_alpha.IsMatch(line))
                        alpha = Convert.ToInt32(re_alpha.Match(line).Groups[1].ToString());

                    if (re_collide.IsMatch(line))
                        collide = Convert.ToBoolean(re_collide.Match(line).Groups[1].ToString());

                    if (re_animate.IsMatch(line))
                        animate = Convert.ToBoolean(re_animate.Match(line).Groups[1].ToString());

                    if (re_animatedelay.IsMatch(line))
                        animated = Convert.ToInt32(re_animatedelay.Match(line).Groups[1].ToString()) >> 1;

                    if (re_position.IsMatch(line))
                        position = new Point(Convert.ToInt32(re_position.Match(line).Groups[1].ToString()), Convert.ToInt32(re_position.Match(line).Groups[2].ToString()));
                }

                if (id != null && name != null)
                {
                    string obj_dir = BoO_Config.Main.PROGRAM_ROOT_PATH + "/" + BoO_Config.Main.level_dir + "/" + level.name + "/obj/" + name;
                    SurfaceCollection sco = new SurfaceCollection();

                    if (Directory.Exists(obj_dir) == false)
                    {
                        Events.QuitApplication();
                    }

                    DirectoryInfo di_o = new DirectoryInfo(obj_dir);

                    int i = 0;
                    foreach (FileInfo fi_o in di_o.GetFiles())
                    {
                        Surface s_o = new Surface(fi_o.FullName).Convert(Video.Screen);
                        if (alpha < 255)
                        {
                            s_o.AlphaBlending = true;
                            s_o.Alpha = Convert.ToByte(alpha);
                        }
                        s_o.TransparentColor = BoO_Config.Video.transparent_color;
                        s_o.Transparent = true;
                        sco.Add(s_o);
                        if (i > 0)
                            sco.Add(s_o);

                        i++;
                    }

                    if (repeat_direction == null)
                    {
                        objStatic oit = new objStatic(collide, position);
                        oit.set_anim("default", sco, animated);
                        oit.set_cur_anim("default");
                        oit.set_anim(animate);

                        set_objStatic(id, oit);
                    }
                    else
                    {
                        SortedDictionary<string, objStatic> oits = new SortedDictionary<string, objStatic>();

                        for (int j = 0; j < repeat_amount; j++)
                        {
                            Point tmp_position = position;
                            if (repeat_direction == "right") tmp_position.X += j * sco[0].Width;
                            else if (repeat_direction == "left") tmp_position.X -= j * sco[0].Width;
                            else if (repeat_direction == "down") tmp_position.Y += j * sco[0].Height;
                            else if (repeat_direction == "up") tmp_position.Y -= j * sco[0].Height;

                            objStatic oit = new objStatic(collide, tmp_position);
                            oit.set_anim("default", sco, animated);
                            oit.set_cur_anim("default");
                            oit.set_anim(animate);

                            oits.Add(id + "_" + j, oit);
                        }

                        set_objsStatic(oits);
                    }
                }
            }
            else if (type == "iobject")
            {
                string id = null, name = null, repeat_direction = null;
                Point position = new Point();
                bool collide = true, animate = false;
                int animated = 500, repeat_amount = 0, alpha = 255;

                foreach (string line in data.Split("\n".ToCharArray()))
                {
                    if (re_id.IsMatch(line))
                        id = re_id.Match(line).Groups[1].ToString();

                    if (re_name.IsMatch(line))
                        name = re_name.Match(line).Groups[1].ToString();

                    if (re_repeat.IsMatch(line))
                    {
                        repeat_direction = re_repeat.Match(line).Groups[1].ToString();
                        repeat_amount = Convert.ToInt32(re_repeat.Match(line).Groups[2].ToString());
                    }

                    if (re_collide.IsMatch(line))
                        collide = Convert.ToBoolean(re_collide.Match(line).Groups[1].ToString());

                    if (re_animate.IsMatch(line))
                        animate = Convert.ToBoolean(re_animate.Match(line).Groups[1].ToString());

                    if (re_animatedelay.IsMatch(line))
                        animated = Convert.ToInt32(re_animatedelay.Match(line).Groups[1].ToString()) >> 1;

                    if (re_position.IsMatch(line))
                        position = new Point(Convert.ToInt32(re_position.Match(line).Groups[1].ToString()), Convert.ToInt32(re_position.Match(line).Groups[2].ToString()));

                    if (re_alpha.IsMatch(line))
                        alpha = Convert.ToInt32(re_alpha.Match(line).Groups[1].ToString());
                }

                if (id != null && name != null)
                {
                    string obj_dir = BoO_Config.Main.PROGRAM_ROOT_PATH + "/" + BoO_Config.Main.level_dir + "/" + level.name + "/obj/" + name;
                    SurfaceCollection sco = new SurfaceCollection();

                    if (Directory.Exists(obj_dir) == false)
                    {
                        Events.QuitApplication();
                    }

                    DirectoryInfo di_o = new DirectoryInfo(obj_dir);

                    int i = 0;
                    foreach (FileInfo fi_o in di_o.GetFiles())
                    {
                        Surface s_o = new Surface(fi_o.FullName).Convert(Video.Screen);
                        if (alpha < 255)
                        {
                            s_o.AlphaBlending = true;
                            s_o.Alpha = Convert.ToByte(alpha);
                        }
                        s_o.TransparentColor = BoO_Config.Video.transparent_color;
                        s_o.Transparent = true;
                        sco.Add(s_o);
                        if (i > 0)
                            sco.Add(s_o);

                        i++;
                    }

                    if (repeat_direction == null)
                    {
                        objInteractive oit = new objInteractive(collide, position);
                        oit.set_anim("default", sco, animated);
                        oit.set_cur_anim("default");
                        oit.set_anim(animate);
                        oit.id = id;

                        set_objInteractive(id, oit);
                    }
                    else
                    {
                        SortedDictionary<string, objInteractive> oits = new SortedDictionary<string, objInteractive>();

                        for (int j = 0; j < repeat_amount; j++)
                        {
                            Point tmp_position = position;
                            if (repeat_direction == "right") tmp_position.X += j * sco[0].Width;
                            else if (repeat_direction == "left") tmp_position.X -= j * sco[0].Width;
                            else if (repeat_direction == "down") tmp_position.Y += j * sco[0].Height;
                            else if (repeat_direction == "up") tmp_position.Y -= j * sco[0].Height;

                            objInteractive oit = new objInteractive(collide, tmp_position);
                            oit.set_anim("default", sco, animated);
                            oit.set_cur_anim("default");
                            oit.set_anim(animate);
                            oit.id = id + "_" + j;

                            oits.Add(id + "_" + j, oit);
                        }

                        set_objsInteractive(oits);
                    }
                }
            }
            else if (type == "collide")
            {
                foreach (string line in data.Split("\n".ToCharArray()))
                {
                    if (re_rectangle.IsMatch(line))
                        ud_collides.Add(new Rectangle(Convert.ToInt32(re_rectangle.Match(line).Groups[1].ToString()), Convert.ToInt32(re_rectangle.Match(line).Groups[2].ToString()), Convert.ToInt32(re_rectangle.Match(line).Groups[3].ToString()), Convert.ToInt32(re_rectangle.Match(line).Groups[4].ToString())));
                }

                Console.WriteLine(ud_collides);
            }
        }
        #endregion loadleveldata
    }
}
