using System;
using System.IO;
using System.Drawing;
using System.Reflection;
using System.Collections;
using System.Collections.Generic;

using SdlDotNet.Core;
using SdlDotNet.Graphics;
using SdlDotNet.Graphics.Sprites;
using SdlDotNet.Input;

using BoO.lib.obj.interfaces;

namespace BoO.lib.obj
{
    class objInteractive : ICoord, IButton
    {
        protected AnimatedSprite _anim;
        protected Point _vcoord, _coord;
        protected Rectangle _vrec, _rec;
        protected bool collide;

        protected string _cur_anim = null, _id = null;

        private Dictionary<string, bool> _keys;

        public objInteractive(bool c, Point vc)
        {
            collide = c;
            sett_coord(vc);
            sett_vcoord(vc);

            _anim = new AnimatedSprite();
            _anim.Position = _coord;
        }

        public void set_anim(String anim, SurfaceCollection surcol, int delay)
        {
            AnimationCollection ac = new AnimationCollection();
            ac.Add(surcol);
            ac.Delay = delay;
            _anim.Animations.Add(anim, ac);
        }

        public void set_cur_anim(String anim)
        {
            _anim.CurrentAnimation = anim;
            _cur_anim = anim;

            sett_vcoord(_vcoord);
        }

        public void set_anim(bool anim)
        {
            _anim.Animate = anim;
        }

        public void move(Dictionary<string, bool> keys)
        {
            sett_coord(_vcoord.X - BoO.share.map_coord.X, _vcoord.Y - BoO.share.map_coord.Y);

            _anim.Position = _coord;
            if(collide == true)
                BoO.lib.obj.level.collides.Add(new Rectangle(_vcoord, _anim.Size));
        }

        public void render()
        {
            Video.Screen.Blit(_anim, _coord);
        }

        #region ICoord
        public void update_coord(int x, int y)
        {
            _coord.X += x;
            _coord.Y += y;

            if (_coord != null && _anim != null)
                _rec = new Rectangle(_coord, _anim.Size);
        }

        public void update_coord(Point p)
        {
            update_coord(p.X, p.Y);
        }

        public void update_vcoord(int x, int y)
        {
            _vcoord.X += x;
            _vcoord.Y += y;

            if (_vcoord != null && _anim != null)
                _vrec = new Rectangle(_vcoord, _anim.Size);
        }

        public void update_vcoord(Point p)
        {
            update_vcoord(p.X, p.Y);
        }

        public void sett_coord(int x, int y)
        {
            _coord.X = x;
            _coord.Y = y;

            if(_coord != null && _anim != null)
                _rec = new Rectangle(_coord, _anim.Size);
        }

        public void sett_coord(Point p)
        {
            sett_coord(p.X, p.Y);
        }

        public void sett_vcoord(int x, int y)
        {
            _vcoord.X = x;
            _vcoord.Y = y;

            if (_vcoord != null && _anim != null)
                _vrec = new Rectangle(_vcoord, _anim.Size);
        }

        public void sett_vcoord(Point p)
        {
            sett_vcoord(p.X, p.Y);
        }

        public Point coord
        {
            get { return _coord; }
        }

        public Point vcoord
        {
            get { return _vcoord; }
        }

        public Rectangle rec
        {
            get { return _rec; }
        }

        public Rectangle vrec
        {
            get { return _vrec; }
        }
        #endregion ICoord

        #region IButton
        public void on_button1_press()
        {
            object t = new object();
            try
            {
                Type.GetType("BoO.lib.levels." + level.name).InvokeMember("btn1_" + this._id, BindingFlags.Static | BindingFlags.InvokeMethod | BindingFlags.Public, null, t, new object[0]);
            }
            catch (Exception)
            {
                Console.WriteLine("No method binded for id " + this._id);
            }
        }

        public void on_button2_press()
        {

        }

        public void on_button3_press()
        {

        }
        #endregion IButton

        #region setsgets
        public string id
        {
            get
            {
                return _id;
            }

            set
            {
                _id = value;
            }
        }
        #endregion setsgets
    }
}
