using System;
using System.IO;
using System.Drawing;
using System.Reflection;
using System.Collections;
using System.Collections.Generic;

using SdlDotNet.Core;
using SdlDotNet.Graphics;
using SdlDotNet.Graphics.Sprites;
using SdlDotNet.Input;

using BoO.lib.obj;

namespace BoO_Main
{
    public class Game_start
    {
        private Dictionary<string, bool> keyactions = new Dictionary<string, bool>(4);
        private TextSprite gamelabel;

        private level l1;

        [STAThread]
        public static void Main()
        {
            Game_start app = new Game_start();
            app.Start();
        }

        public Game_start()
        {
            Video.SetVideoMode(BoO_Config.Video.res_h, BoO_Config.Video.res_v, 32, false, false, BoO_Config.Video.fullscreen, false, true);
            Video.WindowCaption = "RPG unknown <dev>";

            this._init_main();
            this._init_game();
            this._init_screen();
        }

        public void Start()
        {
            Events.Fps = BoO_Config.Main.standard_fps;
            Events.KeyboardDown += new EventHandler<KeyboardEventArgs>(this.eKeyDown);
            Events.KeyboardUp += new EventHandler<KeyboardEventArgs>(this.eKeyUp);
            Events.Tick += new EventHandler<TickEventArgs>(this.eTick);
            Events.Quit += new EventHandler<QuitEventArgs>(this.eQuit);
            Events.Run();
        }

        private void _init_main()
        {
            string platform;

            System.OperatingSystem osi = System.Environment.OSVersion;
            string os = osi.Platform.ToString();

            switch (os)
            {
                case "Win32NT":
                    platform = "WINDOWS"; break;
                case "Unix":
                    platform = "UNIX"; break;
                default:
                    platform = "UNKNOWN"; break;
            }

            BoO_Config.Main.OS = platform;

            System.Reflection.Assembly a = System.Reflection.Assembly.GetEntryAssembly();
            BoO_Config.Main.PROGRAM_ROOT_PATH = Path.GetDirectoryName(a.Location);

            this.keyactions.Add("up", false);
            this.keyactions.Add("down", false);
            this.keyactions.Add("left", false);
            this.keyactions.Add("right", false);

            this.keyactions.Add("button_1", false);
        }

        private void _init_game()
        {
            l1 = new level("start");
        }

        private void _init_screen()
        {
            this.gamelabel = new TextSprite("SDL-RPG-Engine v 0.1.1 CS <dev> on " + BoO_Config.Main.OS, new SdlDotNet.Graphics.Font("data/font/LiberationSans-Regular.ttf", 13));
            this.gamelabel.Position = new Point(BoO_Config.Video.res_h - this.gamelabel.Width - 10, 10);
            Video.Screen.Blit(this.gamelabel, new Point(BoO_Config.Video.res_h - this.gamelabel.Width - 10, 10));
        }

        #region EventHandler Methods

        void eKeyUp(object sender, KeyboardEventArgs e)
        {
            // Cursor left
            if (e.Scancode == 75)
                this.keyactions["left"] = false;

            // Cursor right
            if (e.Scancode == 77)
                this.keyactions["right"] = false;

            // Cursor up
            if (e.Scancode == 72)
                this.keyactions["up"] = false;

            // Cursor down
            if (e.Scancode == 80)
                this.keyactions["down"] = false;
        }

        void eKeyDown(object sender, KeyboardEventArgs e)
        {
            // Escape
            if (e.Scancode == 1)
                Events.QuitApplication();

            // Cursor left
            if (e.Scancode == 75)
                this.keyactions["left"] = true;

            // Cursor right
            if (e.Scancode == 77)
                this.keyactions["right"] = true;

            // Cursor up
            if (e.Scancode == 72)
                this.keyactions["up"] = true;

            // Cursor down
            if (e.Scancode == 80)
                this.keyactions["down"] = true;

            // Action1 
            if (e.Scancode == 30)
                this.keyactions["button_1"] = true;

            Console.WriteLine(e.Scancode);
        }

        private void eTick(object sender, TickEventArgs e)
        {
            // Forward key inputs do level object
            l1.keys = this.keyactions;

            // First update update/compute all objects
            l1.update_background();
            l1.update_main();
            l1.update_objsStatic();
            l1.update_objsInteractive();
            l1.update_characters();
            l1.update_objsAbove();
            
            // All objects are updatet now, time to render
            l1.render_background();
            l1.render_objsStatic();
            l1.render_objsInteractive();
            l1.render_characters();
            l1.render_objsAbove();
            
            Video.Screen.Blit(this.gamelabel);

            Video.Screen.Update();

            BoO.share.frame++;
            BoOevents.events.bg_update = false;

            reset_keys(new string[] { "button_1" });
        }

        private void reset_keys(string[] keys)
        {
            for (int i = 0; i < keys.Length; i++)
            {
                this.keyactions[keys[i]] = false;
            }
        }

        private void eQuit(object sender, QuitEventArgs e)
        {
            Events.QuitApplication();
        }

        #endregion EventHandler Methods
    }
}